@@ -352,6 +352,8 @@ sub on_apply_lst_clicked {
     }
 
     my $dman = $self->{grid}->get_data_manager;
+   
+
     my %old  = (
         countryid => $dman->get_field('countryid'),
         langid    => $dman->get_field('langid')
@@ -370,6 +372,11 @@ sub on_apply_lst_clicked {
 #passing the pk names to apply excludes these from being saved here
 #since they are saved with the hashref  pass to DM->save
     $self->{grid}->apply( \@pks );
+# on which row are we in the grid ?
+ my $row = $self->{grid}->get_current_row;
+    $self->{log}->debug("row pos from grid: ", $row);
+ # set dman to this row before saving, since dman is now positionned on the last row
+    $dman->set_row_pos($row);
     $dman->save(%h);
 
     # $self->{sf_list}->apply(\@pks);
