@@ -13,9 +13,8 @@ use Carp qw(croak confess carp);
 use Data::Dumper;
 use Wx qw(:everything);
 use base qw(Wx::Grid);
-
-use Wx::Event
-  qw( EVT_GRID_CELL_CHANGED EVT_COMBOBOX EVT_GRID_RANGE_SELECT EVT_GRID_SELECT_CELL);
+use Gtk2::Ex::DbLinker::DatasheetHelper;
+use Wx::Event qw( EVT_GRID_CELL_CHANGED EVT_COMBOBOX EVT_GRID_RANGE_SELECT EVT_GRID_SELECT_CELL);
 
 my %signals = ( 'Wx::GridCellChoiceEditor' => \&EVT_COMBOBOX, );
 
@@ -54,20 +53,19 @@ sub new {
 
     #my ($class, $frame, $req) = @_;
     my $class = shift;
-    my %def   = ( borders_size => [ 2, 20 ], fields => undef );
+    my %def   = ( borders_size => [ 20, 20 ], fields => undef );
     my %arg   = ref $_[0] eq "HASH" ? ( %def, %{ $_[0] } ) : ( %def, @_ );
     my $self  = $class->SUPER::new( $arg{parent_widget}, wxID_ANY );
 
     $self->{dman} = $arg{data_manager};
 
     #$self->{fields} = $$req{fields}  || undef;
-    $self->{fields} = $arg{fields};
-    $self->{on_changed} =
-      $arg{on_changed}; # Code that runs when a record is changed ( any column )
+    $self->{fields}        = $arg{fields};
+    $self->{on_changed}    = $arg{on_changed};      # Code that runs when a record is changed ( any column )
     $self->{on_row_select} = $arg{on_row_select};
     $self->{after_update}  = $arg{after_update};
 
-#$self->{borders_size} = $$req{borders_size} || [22, 20]; #array ref of row1 height and col1 width
+    #$self->{borders_size} = $$req{borders_size} || [22, 20]; #array ref of row1 height and col1 width
     $self->{borders_size} = $arg{borders_size};
 
     $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
@@ -89,11 +87,19 @@ sub new {
     $self->_setup_grid;
     $self->SetSelectionMode(wxGridSelectRows);
 
-    EVT_GRID_CELL_CHANGED( $self,
-        sub { my $self = shift; $self->_changed(@_); } );
+    EVT_GRID_CELL_CHANGED( $self, sub { my $self = shift; $self->_changed(@_); } );
     EVT_GRID_SELECT_CELL( $self, sub { shift->_cell_clicked(@_); } );
     EVT_GRID_RANGE_SELECT( $self, sub { shift->_row_selected(@_); } );
     $self->{log}->debug( "constructor done: rows: " . $self->GetNumberRows );
+	
+    $self->{ds_helper} = Gtk2::Ex::DbLinker::DatasheetHelper->new(
+        cols       => $self->{cols},
+        dman       => $self->{dman},
+	col_number => sub { $self->colnumber_from_name($_[0]);}
+	
+
+    );
+#col_number => \&{ $self->colnumber_from_name }
     $self->{changed} = 0;
     return $self;
 
@@ -101,8 +107,7 @@ sub new {
 
 sub _setup_gridtable {
     my $self = shift;
-    return Wx::Perl::DbLinker::DBGridTable->new(
-        { dman => $self->{dman}, fields => $self->{fields}, grid => $self } );
+    return Wx::Perl::DbLinker::DBGridTable->new( { dman => $self->{dman}, fields => $self->{fields}, grid => $self } );
 
 }
 
@@ -116,8 +121,7 @@ sub _setup_fields {
         my $field_percentage =
           $no_of_fields < 8
           ? 100 / $no_of_fields
-          : 12.5
-          ;    # Don't set percentages < 12.5 - doesn't really work so well ...
+          : 12.5;    # Don't set percentages < 12.5 - doesn't really work so well ...
         for my $field ( @{ $self->{cols} } ) {
             my $gtktype = $fieldtype{ $self->{dman}->get_field_type($field) };
             push @{ $self->{fields} },
@@ -126,15 +130,13 @@ sub _setup_fields {
                 x_percent => $field_percentage,
                 renderer  => $gtktype,
               };
-            $self->{log}
-              ->debug( " * set field : " . $field . " renderer : " . $gtktype );
+            $self->{log}->debug( " * set field : " . $field . " renderer : " . $gtktype );
         }
     }
 
     my $column_no = 0;
     for my $field ( @{ $self->{fields} } ) {
-        $self->{log}->debug( "field name : " . $field->{name},
-            " col_number: ", $column_no );
+        $self->{log}->debug( "field name : " . $field->{name}, " col_number: ", $column_no );
 
         $self->{colname_to_number}->{ $field->{name} } = $column_no;
         if ( defined $field->{renderer} && $field->{renderer} eq "hidden" ) {
@@ -143,22 +145,18 @@ sub _setup_fields {
 
         if ( !$field->{renderer} ) {
 
-#my $x = ( $fieldtype{ $self->{fieldsType}->{$field->{name}}}  ?  $fieldtype{$self->{fieldsType}->{$field->{name}}} : "text");
+            #my $x = ( $fieldtype{ $self->{fieldsType}->{$field->{name}}}  ?  $fieldtype{$self->{fieldsType}->{$field->{name}}} : "text");
             my $ftype = $self->{dman}->get_field_type( $field->{name} );
             my $x = ( $fieldtype{$ftype} ? $fieldtype{$ftype} : "text" );
             $field->{renderer} = $x;
-            $self->{log}->debug(
-                "reset renderer for field " . $field->{name} . " to " . $x );
+            $self->{log}->debug( "reset renderer for field " . $field->{name} . " to " . $x );
 
         }
 
         #if ($field->{renderer} eq "combo") {
         #    $self->setup_combo($field->{name});
         #}
-        $self->{log}->debug( " ** set field : "
-              . $field->{name}
-              . " renderer : "
-              . $field->{renderer} );
+        $self->{log}->debug( " ** set field : " . $field->{name} . " renderer : " . $field->{renderer} );
         $field->{column} = $column_no++;
     }
 
@@ -169,7 +167,7 @@ sub _setup_fields {
 #combo are backed up with two array ref and a scalar:
 #$self->{combo_val}->{ $field_name } : array ref of values diplayed
 #self->{combo_id}->{ $field_name } : array ref of corresponding id (that are stored in the table in the $field_name column
-#$self->{combo_cur}->{ $field_name } : an array ref of the id the values currently displayed in the combo
+#$self->{combo_cur} : an array ref of the id the values currently displayed by the combos in the grid
 #
 sub _setup_grid {
     my ($self) = @_;
@@ -193,22 +191,19 @@ sub _setup_grid {
             $field->{editor} = $render{ $field->{renderer} }();
         } elsif ( $field->{renderer} eq "combo" ) {
 
-            $field->{editor} = $render{ $field->{renderer} }
-              ( $self->_setup_combo( $field->{name} ), 0 );
+            $field->{editor} = $render{ $field->{renderer} }( $self->_setup_combo( $field->{name} ), 0 );
 
-#$field->{editor} = Wx::GridCellChoiceEditor->new(  $self->_setup_combo($field->{name}),0 );
+            #$field->{editor} = Wx::GridCellChoiceEditor->new(  $self->_setup_combo($field->{name}),0 );
 
-#$self->SetCellRenderer($row, $col, $ed);
-#$renderer->set(  editable => 1, text_column     => 1,	has_entry   => 0,  model => $model );
-# $renderer->{col_data} = $lastcol++ ;
-            $self->{log}
-              ->debug( "field name with combo renderer: " . $field->{name} );
+            #$self->SetCellRenderer($row, $col, $ed);
+            #$renderer->set(  editable => 1, text_column     => 1,	has_entry   => 0,  model => $model );
+            # $renderer->{col_data} = $lastcol++ ;
+            $self->{log}->debug( "field name with combo renderer: " . $field->{name} );
 
             my $signal = $signals{ ref $field->{editor} };
 
             #http://sourceforge.net/p/wxperl/mailman/message/8872987/
-            &$signal( $self, -1,
-                sub { shift; $self->_combo_edited( $field->{name}, @_ ); } );
+            &$signal( $self, -1, sub { shift; $self->_combo_edited( $field->{name}, @_ ); } );
 
             #  $renderer->{col_data} = $lastcol++ ;
             #
@@ -231,8 +226,7 @@ sub _setup_grid {
 =cut
 
         } else {
-            $self->{log}
-              ->debug( "field name with txt renderer: " . $field->{name} );
+            $self->{log}->debug( "field name with txt renderer: " . $field->{name} );
             $field->{editor} = $render{ $field->{renderer} }();
 
             if ( grep /^$field->{name}$/, @apk ) {
@@ -247,19 +241,21 @@ sub _setup_grid {
          # die ($self->GetNumberRows);
          # enlever -2 a cause des labels
     for my $r ( 0 .. $self->GetNumberRows() - 1 ) {    # Row Header Text
-            #my $rptr = $r+1;
-            #$grid->SetRowLabelValue($r, "Row $rptr");
-            #$grid->SetRowLabelBackgroundColour($r, wxGREEN);
+                                                       #my $rptr = $r+1;
+                                                       #$grid->SetRowLabelValue($r, "Row $rptr");
+                                                       #$grid->SetRowLabelBackgroundColour($r, wxGREEN);
         $self->SetRowLabelValue( $r, (STATUS_LAB)[UNCHANGED] );
 
         #$grid->SetReadOnly($r, 0);
         for my $field ( @{ $self->{fields} } ) {
             my $col = $self->{colname_to_number}->{ $field->{name} };
             if ( $r == 0 ) {
+
                 my $label = (
                     defined $field->{header_markup}
                     ? $field->{header_markup}
-                    : $field->{name} );
+                    : $field->{name}
+                );
                 $self->{log}->debug( "col: ", $col, " label: ", $label );
                 $self->{log}->debug( "cell editor: " . ref $field->{editor} );
                 $self->SetColLabelValue( $col, $label );
@@ -271,9 +267,9 @@ sub _setup_grid {
             if ( $ed_ref eq "Wx::GridCellChoiceEditor" ) {
                 $self->SetCellEditor( $r, $col, $field->{editor} );
 
-# remplacer la valeur de id_credit par le text correspondant dans la combo
-# _find_index est appele pour toutes les lignes de la col, si la liste est longue, ce n'est pas efficace
-# $id est la valeur affichee dans la grid qui provient de la table
+                # remplacer la valeur de id_credit par le text correspondant dans la combo
+                # _find_index est appele pour toutes les lignes de la col, si la liste est longue, ce n'est pas efficace
+                # $id est la valeur affichee dans la grid qui provient de la table
                 my $id = $self->GetCellValue( $r, $col );
 
                 # $val est le array ref des id
@@ -291,11 +287,12 @@ sub _setup_grid {
                     $text = "";
                 }
 
-# combo_cur contient l'id de la derniere ligne affichee de la grid
-# $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
-#$self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
-#$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
-                my $aref = $self->{combo_cur}->{ $field->{name} };
+                # combo_cur contient l'id de la derniere ligne affichee de la grid
+                # $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
+                #$self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
+                #$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
+                #my $aref = $self->{combo_cur}->{ $field->{name} };
+                my $aref = $self->{combo_cur};
                 push @$aref, $id;
                 $self->SetCellOverflow( $r, $col, 0 );
                 $self->SetCellValue( $r, $col, $text );
@@ -306,13 +303,12 @@ sub _setup_grid {
                 $self->SetCellRenderer( $r, $col, $field->{editor} );
                 if ( $field->{renderer_function} ) {
                     my $coderef = $field->{renderer_function};
-                    $self->SetCellValue( $r, $col,
-                        &$coderef( $r, $col, $self->GetTable ) );
+                    $self->SetCellValue( $r, $col, &$coderef( $r, $col, $self->GetTable ) );
                 }
 
             }
 
-#$self->{log}->debug("set cell renderer ", $ed_ref, " row: ", $r, " col: ", $col);
+            #$self->{log}->debug("set cell renderer ", $ed_ref, " row: ", $r, " col: ", $col);
 
             if ( $field->{editor_ro} ) {
                 $self->SetReadOnly( $r, $col, 1 );
@@ -338,10 +334,7 @@ sub _setup_combo {
     my ( $self, $fieldname ) = @_;
 
     my $column_no = $self->{colname_to_number}->{$fieldname};
-    $self->{log}->debug( "setup_combo field name : "
-          . $fieldname
-          . " col number :"
-          . $column_no );
+    $self->{log}->debug( "setup_combo field name : " . $fieldname . " col number :" . $column_no );
 
     #my @combo = @{$self->{fields}[$column_no]->{data}};
     my $dman = $self->{fields}[$column_no]->{data_manager};
@@ -383,7 +376,7 @@ sub _setup_combo {
             push @rows, $rowval if ( $pos == $lastcol );
             $pos++;
 
-  # $self->{log}->debug("field: " . $name . " val : ". $dman->get_field($name));
+            # $self->{log}->debug("field: " . $name . " val : ". $dman->get_field($name));
         }
 
         # print Dumper(@model_row);
@@ -392,11 +385,13 @@ sub _setup_combo {
 
     }    #for
 
-#  fieldname est le nom du champ li au combo dans la table affichee par la grid
-#  il est unique pour la table contrairement aux champs qui sont dans la combo
+    #  fieldname est le nom du champ li au combo dans la table affichee par la grid
+    #  il est unique pour la table contrairement aux champs qui sont dans la combo
     $self->{combo_id}->{$fieldname}  = \@ids;
     $self->{combo_val}->{$fieldname} = \@rows;
-    $self->{combo_cur}->{$fieldname} = [];
+
+    #$self->{combo_cur}->{$fieldname} = [];
+    $self->{combo_cur} = [];
     return \@rows;
 
 }    #setup_combo
@@ -408,8 +403,7 @@ sub get_column_value {
     my ( $self, $sql_fieldname ) = @_;
 
     my $current_row = $self->{current_row};
-    $self->{log}->debug( "get_column_value current row : "
-          . ( defined $current_row ? $current_row : " undef" ) );
+    $self->{log}->debug( "get_column_value current row : " . ( defined $current_row ? $current_row : " undef" ) );
 
     # my $model = $self->{treeview}->get_model;
 
@@ -420,8 +414,10 @@ sub get_column_value {
 
     if ( $self->_is_combo($column_no) ) {
 
-        my $aref = $self->{combo_cur}->{$sql_fieldname};
+        #my $aref = $self->{combo_cur}->{$sql_fieldname};
+        my $aref = $self->{combo_cur};
         $value = $aref->[$current_row];
+        $self->{log}->debug( "get_column_value combo_cur : ", Dumper $aref);
         $self->{log}->debug( "get_column_value is combo return  : " . $value );
     } else {
 
@@ -437,14 +433,13 @@ sub set_column_value {
 
     my $done;
 
-#if ( $self->{mult_select} ) {
-#    $self->{log}->debug("set_column_value called with multi_select enabled -> setting value in 1st selected row");
-#}
+    #if ( $self->{mult_select} ) {
+    #    $self->{log}->debug("set_column_value called with multi_select enabled -> setting value in 1st selected row");
+    #}
 
     #my  @selected_rows = $self->GetSelectedRows;
     my $current_row = $self->{current_row};
-    $self->{log}->debug( "set_column_value current row : "
-          . ( defined $current_row ? $current_row : " undef" ) );
+    $self->{log}->debug( "set_column_value current row : " . ( defined $current_row ? $current_row : " undef" ) );
 
     #if ( ! scalar( @selected_rows ) ) {
     #    return 0;
@@ -467,9 +462,14 @@ sub set_column_value {
 
 }
 
+sub get_current_row {
+    return shift->{current_row};
+}
+
 sub colnumber_from_name {
 
     my ( $self, $fieldname ) = @_;
+    confess "fieldname undef" unless ( defined $fieldname);
     return $self->{colname_to_number}->{$fieldname}
 
 }
@@ -508,8 +508,8 @@ sub update {
     #$self->ForceRefresh;
     my $table = $self->_setup_gridtable;
 
-#	Wx::Perl::DbLinker::DBGridTable->new({dman=> $self->{dman}, fields => $self->{fields}, grid=>$self,});
-#$self->{log}->debug("update : ",join(" ", @));
+    #	Wx::Perl::DbLinker::DBGridTable->new({dman=> $self->{dman}, fields => $self->{fields}, grid=>$self,});
+    #$self->{log}->debug("update : ",join(" ", @));
     $self->SetTable($table);
     $self->_setup_grid;
     if ( defined $self->{after_update} ) {
@@ -520,6 +520,60 @@ sub update {
 }
 
 sub apply {
+    my $self  = shift;
+    my $pkref = $_[0];
+    my $last  = $self->GetNumberRows;
+
+    #$self->{ds_helper}->init_rows($self);
+    $self->{ds_helper}->init(
+        iter         => 0,
+        next         => sub { my $p = $_[0]; $p++; return ( $p < $last ? $p : undef ); },
+        has_more_row => sub { my $v = ( $self->{iter} < $last ); $self->{log}->debug( "has_more_row ", ( $v ? " true" : " false" ) ); return $v; },
+        get_val      => sub {
+            my ( $row, $col, $name ) = @_;
+            my $val;
+            if ( $col == 0 ) {
+                my $status_lab = $self->GetRowLabelValue($row);
+
+                $val = $self->_find_index( $status_lab, (STATUS_LAB) );
+            } else {
+                if ( $self->_is_combo($col) ) {
+                    my $aref = $self->{combo_cur}->{$name};
+                    $val = $aref->[$row];
+                    $self->{log}->debug( " found " . $val . " for grid row " . $row . " for combo " . $name );
+                } else {
+                    $val = $self->GetTable->GetValue( $row, $col );
+                }
+            }
+            $self->{log}->debug( "get_val found: ", ( defined $val ? $val : "undef" ), " for row : ", $row, " col : ", $col );
+            return $val;
+        },
+        set_val => sub {
+            my ( $row, $col, $val ) = @_;
+            $self->{log}->debug( "set_val : ", ( defined $val ? $val : " undef" ), " for row : ", $row, " col : ", $col );
+            if ( $col == 0 ) {
+                my $label = (STATUS_LAB)[$val];
+                $self->{log}->debug( "label found : ", $label );
+                $self->SetRowLabelValue( $row, $label );
+            } else {
+                $self->SetCellValue( $row, $col, $val );
+            }
+        },
+        del_row => sub {
+            $self->{log}->debug( "deleting at row ", $_[0] );
+            $self->DeleteRows( $_[0] );
+        }
+    );
+
+    #$self->{ds_helper}->{dman} = $self->{dman};
+    $self->{ds_helper}->apply($pkref);
+
+    $self->{after_update}() if ( defined $self->{after_update} );
+
+}
+
+=for comment
+sub apply_old {
 
     my $self = shift;
 
@@ -605,7 +659,8 @@ sub apply {
                     my $col = $self->{colname_to_number}->{$name};
                     my $x;
                     if ( $self->_is_combo($col) ) {
-                        my $aref = $self->{combo_cur}->{$name};
+			#my $aref = $self->{combo_cur}->{$name};
+			my $aref = $self->{combo_cur};
                         $x = $aref->[$row_pos];
                         $self->{log}->debug( " found "
                               . $x
@@ -668,6 +723,7 @@ sub apply {
     }
 
 }    #apply
+=cut
 
 sub insert {
 
@@ -683,12 +739,11 @@ sub insert {
 
     #push @new_record,        $iter,        STATUS_COLUMN,        INSERTED;
 
-    $self->{log}
-      ->debug( "new rec default values: " . join( " ", @columns_and_values ) );
+    $self->{log}->debug( "new rec default values: " . join( " ", @columns_and_values ) );
 
     #$model->set( @new_record );
 
-#$self->{treeview}->set_cursor( $model->get_path($iter), $self->{fields}[0]->{treeview_column}, 0 );
+    #$self->{treeview}->set_cursor( $model->get_path($iter), $self->{fields}[0]->{treeview_column}, 0 );
     $self->AppendRows(1);
     my $last = $self->GetNumberRows - 1;
 
@@ -709,12 +764,7 @@ sub insert {
                 #die ($col_no);
                 $i = 0;
             } else {
-                $self->{log}->debug( "last: "
-                      . $last
-                      . " col: "
-                      . $col_no
-                      . " value: "
-                      . $col_no_or_value );
+                $self->{log}->debug( "last: " . $last . " col: " . $col_no . " value: " . $col_no_or_value );
                 $self->SetCellValue( $last, $col_no, $col_no_or_value );
                 $i = 1;
             }
@@ -752,8 +802,7 @@ sub _row_selected {
     my ( $self, $e ) = @_;
     my $top = $e->GetTopRow;
     my $bot = $e->GetBottomRow;
-    $self->{log}
-      ->debug( "_row_selected top row: " . $top . " bottom row : " . $bot );
+    $self->{log}->debug( "_row_selected top row: " . $top . " bottom row : " . $bot );
     if ( $top == $bot ) {
         $self->{current_row} = $top;
         $self->{on_row_select}() if ( defined $self->{on_row_select} );
@@ -765,6 +814,8 @@ sub _row_selected {
 
 #called after a change in the combo
 # $combo -> $tree
+# problem : if the first row in the combo is choosen without a click on it
+# the event is not fired and the array combo_cur is not updated
 sub _combo_edited {
 
     #my  ($self, $renderer, $path_string, $new_text) = @_;
@@ -786,79 +837,19 @@ sub _combo_edited {
     # die(Dumper $aref);
     my @a = @{$aref};
     $self->{log}->debug( "id: " . Dumper($id) );
-    $self->{log}->debug( "new value:  "
-          . $new_text
-          . " index: "
-          . $i
-          . " corresponding id : "
-          . $a[$i] );
+    $self->{log}->debug( "new value:  " . $new_text . " index: " . $i . " corresponding id : " . $a[$i] );
 
     #$self->{combo_cur}->{$id } = $a[$i];
 
     #update the array backing the combo column in the grid
 
-    $self->{combo_cur}->{$id}->[$current_row] = $a[$i];
+    #$self->{combo_cur}->{$id}->[$current_row] = $a[$i];
+    $self->{combo_cur}->[$current_row] = $a[$i];
 
     #	$cell->get("model");
 
 }
 
-sub _cell_edited_todel {
-    my ( $self, $cell, $path_string, $new_text ) = @_;
-    my $path  = Gtk2::TreePath->new_from_string($path_string);
-    my $model = $self->{treeview}->get_model;
-    my $col   = $cell->{column};
-    my $iter  = $model->get_iter($path);
-
-    $model->set_value( $iter, $col, $new_text );
-}
-
-sub _toggle_edited_todel {
-    my ( $self, $renderer, $text_path, $something ) = @_;
-    my $column_no = $renderer->{column};
-    my $path      = Gtk2::TreePath->new($text_path);
-    my $model     = $self->{treeview}->get_model;
-    my $iter      = $model->get_iter($path);
-    my $old_value = $model->get( $iter, $renderer->{column} );
-    my $new_text  = !$old_value;
-
-    $model->set( $iter, $renderer->{column}, $new_text );
-}
-
-sub _start_editable_todel {
-    my ( $self, $cell, $editable, $path, $renderer ) = @_;
-    $self->{log}->debug("start_editable");
-
-    # print Dumper($editable);
-    # $maincombo = $editable;
-    $renderer->{combo} = $editable;
-
-}
-
-sub _combo_value_todel {
-    my ( $self, $combo_model, $id, $comp_ref ) = @_;
-    my $iter = $combo_model->get_iter_first();
-    my $key  = -1;
-    my $value;
-
-    # while ($iter && $key != $id){
-    while ( $iter && &$comp_ref( $key, $id, 0 ) ) {
-        $key = $combo_model->get_value( $iter, 0 );
-
-        #  if ($key == $id) {
-        if ( &$comp_ref( $key, $id, 1 ) ) {
-
-            $value = $combo_model->get_value( $iter, 1 );
-            $self->{log}->debug( "found : " . $value . " for " . $id );
-            last;
-        }
-        $iter = $combo_model->iter_next($iter);
-    }
-
-    return $value;
-
-}
-
 sub get_data_manager {
     return shift->{dman};
 }
@@ -930,7 +921,7 @@ sub new {
             my $fname = $f->{name};
             my $v     = $self->{dman}->get_field($fname);
 
- #$self->{log}->debug(" row: ", $i , " $fname: ", (defined $v ? $v : " undef"));
+            #$self->{log}->debug(" row: ", $i , " $fname: ", (defined $v ? $v : " undef"));
             $$self{array}->[$i][ $col++ ] = $v;
         }
 
@@ -949,8 +940,8 @@ sub GetNumberCols {
 sub GetNumberRows {
     my ($self) = @_;
 
-#$$self{log}->debug("GrigTable-GNR called");
-#mais il y a la ligne des labels...mais grid->GetNumberRows ne la compte pas non plus
+    #$$self{log}->debug("GrigTable-GNR called");
+    #mais il y a la ligne des labels...mais grid->GetNumberRows ne la compte pas non plus
     return $self->{rows};
 }
 
@@ -968,9 +959,8 @@ sub GetValue {
     if ( $row < $self->GetNumberRows && $col < $self->GetNumberCols ) {
         my $aref = $$self{array}->[$row];
 
-#$self->{log}->debug("GetValue : " . $row . " " . $col . " array : " . (defined $aref ? " defined " : " undef") );
-        if ( defined $$self{array}->[$row] )
-        {    #retourner $result s'il a ete attribue par SetValue
+        #$self->{log}->debug("GetValue : " . $row . " " . $col . " array : " . (defined $aref ? " defined " : " undef") );
+        if ( defined $$self{array}->[$row] ) {    #retourner $result s'il a ete attribue par SetValue
             $result = $$self{array}->[$row][$col];
         } else {
             confess("array undef");
@@ -978,7 +968,7 @@ sub GetValue {
 
         my $fname = $self->{fields}[$col]->{name};
 
-#$self->{log}->debug("GetValue: ", (defined $result? $result: " undef"), " Field: ", $fname, " row: ", $row, " col: ", $col);
+        #$self->{log}->debug("GetValue: ", (defined $result? $result: " undef"), " Field: ", $fname, " row: ", $row, " col: ", $col);
 
     } else {
         carp( "Index out of bound in DBGridTable: " . $row . " : " . $col );
@@ -993,7 +983,7 @@ sub GetValue {
 sub SetValue {
     my ( $self, $row, $col, $value ) = @_;
 
-#$self->{log}->debug ("SetValue GNR: " . $self->GetNumberRows . " pos to add to is " . $row) ;
+    #$self->{log}->debug ("SetValue GNR: " . $self->GetNumberRows . " pos to add to is " . $row) ;
     confess("Array out of bounds too much rows")
       unless ( $row < $self->GetNumberRows );
     confess("Array out of bounds too much cols")
@@ -1001,12 +991,12 @@ sub SetValue {
 
     my $fname = $self->{fields}->[$col]->{name};
 
-#$self->{log}->debug("SetValue row : ", $row, " col: ", $col, " field: ", $fname , " value: ", (defined $value ? $value : " undef"));
+    #$self->{log}->debug("SetValue row : ", $row, " col: ", $col, " field: ", $fname , " value: ", (defined $value ? $value : " undef"));
 
-#fait dans apply ... !
-#D'autant plus que $value est la valeur affichee par un combo et non l'id correspondant
-#$self->{dman}->set_row_pos($row);
-#    $self->{dman}->set_field($fname, $value);
+    #fait dans apply ... !
+    #D'autant plus que $value est la valeur affichee par un combo et non l'id correspondant
+    #$self->{dman}->set_row_pos($row);
+    #    $self->{dman}->set_field($fname, $value);
 
     $$self{array}->[$row][$col] = $value;
 }
@@ -1167,9 +1157,7 @@ sub AppendRows {
 
         if ( my $grid = $self->GetView ) {
             $self->{log}->debug("AppendRows");
-            my $msg =
-              Wx::GridTableMessage->new( $self,
-                wxGRIDTABLE_NOTIFY_ROWS_APPENDED, $rows_add );
+            my $msg = Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, $rows_add );
             $grid->ProcessTableMessage($msg);
         }
     };
@@ -1216,9 +1204,7 @@ sub DeleteRows {
         $$self{rows} -= $rows;
     }
     if ( my $grid = $self->GetView ) {
-        my $msg =
-          Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_DELETED,
-            $pos, $rows );
+        my $msg = Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_DELETED, $pos, $rows );
         $grid->ProcessTableMessage($msg);
     }
     return 1;
@@ -1233,29 +1219,25 @@ sub setup_empty_row {
         if ( ref $field->{editor} eq "Wx::GridCellChoiceEditor" ) {
             $grid->SetCellEditor( $r, $col, $field->{editor} );
 
-#my $id = $grid->GetCellValue($r, $col);
-# $val est le array ref des id
-#my $val = $grid->{combo_id}->{ $field->{name} };
-#$pos et l'index de id dans ce vecteur
-#my $pos = $self->_find_index($id, @$val);
-# $text est le text a afficher dans la cellule  la place du id
-#my $text = $self->{combo_val}->{$field->{name} }->[$pos];
-# combo_cur contient l'id de la derniere ligne affichee de la grid
-# $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
-# $self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
-#$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
-#my $aref = $self->{combo_cur}->{ $field->{name} };
-#push @$aref, $id;
+            #my $id = $grid->GetCellValue($r, $col);
+            # $val est le array ref des id
+            #my $val = $grid->{combo_id}->{ $field->{name} };
+            #$pos et l'index de id dans ce vecteur
+            #my $pos = $self->_find_index($id, @$val);
+            # $text est le text a afficher dans la cellule  la place du id
+            #my $text = $self->{combo_val}->{$field->{name} }->[$pos];
+            # combo_cur contient l'id de la derniere ligne affichee de la grid
+            # $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
+            # $self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
+            #$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
+            #my $aref = $self->{combo_cur}->{ $field->{name} };
+            #push @$aref, $id;
             $grid->SetCellOverflow( $r, $col, 0 );
 
             #  $self->SetCellValue($r, $col, $text);
         } else {
             $grid->SetCellRenderer( $r, $col, $field->{editor} );
-            $self->{log}->debug(
-                "set cell renderer ",
-                ref $field->{editor},
-                " row: ", $r, " col: ", $col
-            );
+            $self->{log}->debug( "set cell renderer ", ref $field->{editor}, " row: ", $r, " col: ", $col );
 
         }
         if ( $field->{editor_ro} ) {
