@@ -5,7 +5,7 @@ our $VERSION = $Wx::Perl::DbLinker::VERSION;
 
 use strict;
 use warnings;
-use base 'Gtk2::Ex::DbLinker::AbForm';
+use parent 'Gtk2::Ex::DbLinker::AbForm';
 use Carp qw(croak confess carp);
 
 #use DateTime::Format::Strptime;
@@ -1082,7 +1082,7 @@ Sets the value of a data widget from its id
 
 =item C<delete()> See L<Gtk2::Ex::DbLinker::AbForm/delete()>
 
-=item C<apply( [fieldname1, fieldname2 ...] )> See L<Gtk2::Ex::DbLinker::AbForm/apply( [fieldname1, fieldname2 ...] )>
+=item C<apply()> See L<Gtk2::Ex::DbLinker::AbForm/apply()>
 
 =item C<undo()> See L<Gtk2::Ex::DbLinker::AbForm/undo()>
 
